/*
 * File: cdoptions.h
 * Code: Ben Walding
 * Date: 4 November 1997
 *
 * Display and handles options for the cdlabeller
 * Doesn't allow resizing.
 * Protected by the GNU Public License v2.0  <distrib>/GPL.LICENSE
 */

// System includes
#include <stdio.h>
#include <sys/stat.h>
#include <stdlib.h>


// Qt includes
#include <qapp.h>
#include <qpushbt.h>
#include <qlistbox.h>
#include <qfiledlg.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qmsgbox.h>
#include <qmenubar.h>
#include <qradiobt.h>
#include <qsemimodal.h>
#include <qprogdlg.h>
#include <qtimer.h>
#include <qtabdlg.h>
#include <qbttngrp.h>
#include <qlined.h>
#include <qmlined.h>


// Local includes
#include "cdoption.h"
#include "cdlabel.h"
#include "cdoption.moc"

static bool locked = FALSE;
Options global_options; // These are the current options




cdOptions::cdOptions(QWidget *parent, const char *name, bool modal, WFlags f) : QTabDialog(parent,name,modal,f)
{
  // Disallow any resizing
  setMinimumSize(600,300);
  setMaximumSize(600,300);

  locked = TRUE;
  //*********************** TAB STUFF *****************************//
  // Make the tab thing    
  QTabDialog *tab = this;
  tab->setCaption( "Preferences.." );
  
  tab->setOkButton("&Save");
  tab->setApplyButton("&Apply");
  tab->setCancelButton("Cancel");

  connect( tab, SIGNAL(applyButtonPressed()), SLOT(apply()) );
  connect( tab, SIGNAL(cancelButtonPressed()), SLOT(cancel()) );

  
  

  //******************* First page
  page1 = new QWidget(tab, "page1" );
  tab->addTab(page1, "&Printing");


  QButtonGroup *print_group = new QButtonGroup(tab, "output");
  print_group->setGeometry(20, 35, width() - 40, height() - 100);
  print_group->setFrameStyle(QFrame::NoFrame);

  out_lpr = new QRadioButton("Printer", print_group, "out_lpr");
  out_lpr->setGeometry(0, 30, 80, 20);
  cmd_lpr = new QLineEdit(print_group, "cmd_lpr");
  cmd_lpr->setText(global_options.lpr_default);
  cmd_lpr->setGeometry(90, 30, 280, 20);

  lpr_split = new QCheckBox("One file per lpr request", this, "lpr_split");
  lpr_split->setGeometry(410, 65, 180, 20);

  out_file = new QRadioButton("File", print_group, "out_file");
  out_file->setGeometry(0,70, 80, 20);

  cmd_file_front = new QLineEdit(print_group, "cmd_file");
  cmd_file_front->setText(global_options.file_front_default);
  cmd_file_front->setGeometry(90, 70, 280, 20);

  b_browse_front = new QPushButton("Browse...", print_group, "b_browse");
  b_browse_front->setGeometry(390, 70, 100, 20);
  connect(b_browse_front, SIGNAL(clicked()), SLOT(browse_front()));

  cmd_file_back = new QLineEdit(print_group, "cmd_file");
  cmd_file_back->setText(global_options.file_back_default);
  cmd_file_back->setGeometry(90, 100, 280, 20);

  b_browse_back = new QPushButton("Browse...", print_group, "b_browse");
  b_browse_back->setGeometry(390, 100, 100, 20);
  connect(b_browse_back, SIGNAL(clicked()), SLOT(browse_back()));
  connect(print_group, SIGNAL(clicked(int)), SLOT(output_clicked(int)));


  //**************** Interface options page
  pageInterface = new QWidget(tab, "pageInterface");
  tab->addTab(pageInterface, "&Interface");

  QButtonGroup *sort_group = new QButtonGroup(pageInterface, "sort");
  sort_group->setGeometry(20, 35, width() - 40, height() - 100);
  sort_group->setFrameStyle(QFrame::NoFrame);

  sortArtist = new QRadioButton("Sort by Artist", sort_group);
  sortArtist->setGeometry(10, 10, 130, 20);
  sortTitle = new QRadioButton("Sort by Title", sort_group);
  sortTitle->setGeometry(10, 30, 130, 20);

  splash = new QCheckBox("Display &Splash screen on startup", pageInterface, "splash");
  splash->setGeometry(30, 120, 300, 20);
  splash->show();

  wstyle = new QCheckBox("&Motif", pageInterface);
  wstyle->setGeometry(30, 150, 300, 20);
  wstyle->show();

  //******************* Second page
  page2 = new QWidget(tab, "page2");
  tab->addTab(page2, "&Options");

  intermediate = new QCheckBox("&Delete intermediate files", page2, "intermediate");
  intermediate->setGeometry(30,30,300,20);
  
  cdlogo = new QCheckBox("&Add CD-Label logo to inside front insert", page2, "cdlogo");
  cdlogo->setGeometry(30,60,450,20);
  connect(cdlogo, SIGNAL(toggled(bool)), SLOT(changeLogo(bool)));

  
//  Need browser still.
  cdlogo_path = new QLineEdit( page2, "cdlogo_path");
  cdlogo_path->setGeometry(50, 90, 450, 20);
  
  cdlogo_browse = new QPushButton( "Browse...", page2, "cdlogo_browse");
  cdlogo_browse->setGeometry(530, 90, 100, 20);
  connect(cdlogo_browse, SIGNAL(clicked()), SLOT(browseLogo()));


  //******************* Third page
  page3 = new QWidget(tab, "page3");
  tab->addTab(page3, "&Paths");

  QLabel *label_ps = new QLabel("PostScript viewer", page3, "label_dvi");
  label_ps->setGeometry(10,20, 120, 20);
  ps_view = new QLineEdit(page3, "ps_view");
  ps_view->setGeometry(140,20,300,20);
  ps_view->show();

  QLabel *label_fig2dev = new QLabel("fig2dev", page3, "label_fig2dev");
  label_fig2dev->setGeometry(10, 50, 120, 20);
  fig2dev = new QLineEdit(page3, "fig2dev");
  fig2dev->setGeometry(140, 50, 300, 20);
  fig2dev->show();

  QLabel *label_template = new QLabel("Template location", page3, "label_template");
  label_template->setGeometry(10, 80, 120, 20);
  loc_template = new QLineEdit(page3, "loc_template");
  loc_template->setGeometry(140, 80, 300, 20);
  loc_template->show();

  QLabel *label_workmandb = new QLabel("WorkMan database", page3, "workmandb");
  label_workmandb->setGeometry(10, 110, 120, 20);
  workmandb = new QMultiLineEdit(page3, "workmandb");
  workmandb->setGeometry(140, 110, 300, 80);
  workmandb->show();


  connect(this, SIGNAL(changedOptions()), parent, SLOT(changedOptions()));
  connect(this, SIGNAL(changedSortOrder()), parent, SLOT(changedSortOrder()));

  dispOptions();
  //******************* Display TAB
  tab->show();
  tab->setGeometry(0,0,width(),height());


  return;
};

void cdOptions::output_clicked(int) {
  bool value = out_lpr->isChecked();

  cmd_lpr->setEnabled(value);
  cmd_file_front->setEnabled(!value);
  cmd_file_back->setEnabled(!value);
  b_browse_front->setEnabled(!value);
  b_browse_back->setEnabled(!value);
}



// Isn't ever really deleted except when app shuts down..
cdOptions::~cdOptions()
{
  delete ps_view;
  delete page1;
  delete page2;
  delete page3;
  locked = FALSE;  // Redundant?
}


void cdOptions::cancel()
{
}

void cdOptions::apply()
{
  //Printing options
  global_options.out_lpr = out_lpr->isChecked();
  global_options.out_file = out_file->isChecked();
  global_options.lpr_split = lpr_split->isChecked();

  strcpy(global_options.lpr_default, cmd_lpr->text()); 
  strcpy(global_options.file_front_default, cmd_file_front->text());
  strcpy(global_options.file_back_default, cmd_file_back->text());
  

  //Utility options
  strcpy(global_options.ps_view, ps_view->text()); 
  strcpy(global_options.fig2dev, fig2dev->text());
  strcpy(global_options.loc_template, loc_template->text());
  strcpy(global_options.workmandb, workmandb->text());

  //Options
  global_options.cdlogo = cdlogo->isChecked();

  if (global_options.cdlogo)
	strcpy(global_options.cdlogo_path, cdlogo_path->text());

  global_options.delete_intermediate = intermediate->isChecked();
  global_options.splash = splash->isChecked();
  global_options.motifstyle = wstyle->isChecked();
  app->setStyle(global_options.motifstyle?MotifStyle:WindowsStyle);

  //Interface options
  if (global_options.sortartist != sortArtist->isChecked()) {
	global_options.sortartist = sortArtist->isChecked();
	emit changedSortOrder(); //To let the main list update itself
  }
  global_options.sortartist = sortArtist->isChecked();
  saveOptions();
  return;
};


// Isn't ever really deleted except when app shuts down..



// Ignores the value, which makes it easier on non signal callers.
void cdOptions::changeLogo(bool) {
  // Make things visible or invisible
  cdlogo_path->setEnabled(cdlogo->isChecked());
  cdlogo_browse->setEnabled(cdlogo->isChecked());

}

void cdOptions::browseLogo(){
  QString f = QFileDialog::getOpenFileName( 0, "*.gif", this );
  if ( !f.isEmpty() ) {
    cdlogo_path->setText(f);
  } else {
    return;
  }
}


void cdOptions::dispOptions()
{
  out_lpr->setChecked(global_options.out_lpr);
  out_file->setChecked(global_options.out_file);
  lpr_split->setChecked(global_options.lpr_split);
  ps_view->setText(global_options.ps_view);
  cmd_lpr->setText(global_options.lpr_default);
  cmd_file_front->setText(global_options.file_front_default);
  cmd_file_back->setText(global_options.file_back_default);
  fig2dev->setText(global_options.fig2dev);
  intermediate->setChecked(global_options.delete_intermediate);
  loc_template->setText(global_options.loc_template);
  workmandb->setText(global_options.workmandb);
  cdlogo->setChecked(global_options.cdlogo);
  cdlogo_path->setText(global_options.cdlogo_path);
  splash->setChecked(global_options.splash);
  wstyle->setChecked(global_options.motifstyle);
  output_clicked(0);

  if (global_options.sortartist == TRUE)
	sortArtist->setChecked(TRUE);
  else
	sortTitle->setChecked(TRUE);

  changeLogo(FALSE); 
  return;
}

void cdOptions::loadOptions()
{
  read_rc();
}

void cdOptions::browse_front()
{
  QString f = QFileDialog::getOpenFileName( 0, "*.ps", this );
  if ( !f.isEmpty() ) {
    cmd_file_front->setText(f);
  } else {
    return;
  }
}

void cdOptions::browse_back()
{
  QString f = QFileDialog::getOpenFileName( 0, "*.ps", this );
  if ( !f.isEmpty() ) {
    cmd_file_back->setText(f);
  } else {
    return;
  }
}

void save_rc(void);

void cdOptions::saveOptions() {
  emit changedOptions();
  save_rc();
};
  


typedef struct rclist RCLIST;

struct rclist {
  char *label;
  int	  type;
  char *default_entry;
  void *var;
};

#define MAX_OPTIONS 20

#define VAL_ONOFF   0
#define VAL_INT	    1
#define VAL_STRING  2

int linenumber = 0;

const RCLIST options[MAX_OPTIONS] = {
  { "out_lpr",        VAL_ONOFF,  "off",       &(global_options.out_lpr) },
  { "out_file",       VAL_ONOFF,  "on",        &(global_options.out_file) },
  { "lpr_split",      VAL_ONOFF,  "off",       &(global_options.lpr_split) },
  { "ps_view",        VAL_STRING, "ghostview", &(global_options.ps_view) },
  { "file_front__default",   VAL_STRING, "cdlabel-front.ps",   &(global_options.file_front_default) },
  { "file_back__default",   VAL_STRING, "cdlabel-back.ps",   &(global_options.file_back_default) },
  { "lpr_default",    VAL_STRING, "lpr",       &(global_options.lpr_default) },
  { "fig2dev",        VAL_STRING, "fig2dev",   &(global_options.fig2dev) },
  { "delete_intermediate", VAL_ONOFF, "on",    &(global_options.delete_intermediate) },
  { "loc_template",   VAL_STRING, "/usr/local/lib/cdlabel", &(global_options.loc_template)},
  { "workmandb",      VAL_STRING, "~/.workmandb", &(global_options.workmandb)},
  { "cdlogo",         VAL_ONOFF,  "on", &(global_options.cdlogo)},
  { "cdlogo_path",    VAL_STRING, "/usr/local/lib/cdlogo-1.gif", &(global_options.cdlogo_path)},
  { "splash",         VAL_ONOFF,  "on",        &(global_options.splash)},
  { "sortartist",     VAL_ONOFF,  "on",        &(global_options.sortartist)},
  { "motifstyle",     VAL_ONOFF,  "off",       &(global_options.motifstyle)},
  { NULL, 0, NULL, NULL }
};

//int set_options[MAX_OPTIONS];//Whats this for XXX XXX XXX


char *get_next_line(FILE *input)
{
  static char buf[1024];
  char *ptr;
  char *ptrNew;
  
  
  if (!fgets(buf, sizeof(buf), input)) 
    return NULL;

  ptr = buf;
  if (*ptr == '\n') *ptr = 0;
  if (*ptr == '\r') *ptr = 0;
  
  /* Remove comments */
  ptr = strstr(buf, "#");
  if (ptr)
    *ptr = 0;
  
  ptr = buf;

  for (ptrNew = ptr; *ptrNew != '\0'; ptrNew++) {
	if (*ptrNew == '~') {
	  char buf_new[128];
	  sprintf(buf_new, "%s%s", getenv("HOME"), (ptrNew + 1));
	  strcpy(ptrNew, buf_new); /* Messy, but who cares */
	}
  }
  return (ptr);
}

void read_rc(void)
{
  FILE *rcfile;
    char buf[256];
    char *ptr;
    char field[128];
    char argument[128];
    int iline;


    sprintf(buf, "%s/.cdlabelrc", getenv("HOME"));
    /* Install defaults */
    for (iline = 0; options[iline].label != NULL; iline++) {
      switch (options[iline].type) {
      case VAL_ONOFF:
		if (!strcasecmp(argument, options[iline].default_entry)) 
		  *((bool*) options[iline].var) = TRUE;
		else
		  *((bool*) options[iline].var) = FALSE;
		break;
		
      case VAL_INT:
		*((int*) options[iline].var) = atoi(options[iline].default_entry);
		break;
		
      case VAL_STRING:
		strcpy((char *)options[iline].var, options[iline].default_entry);
		break;
      }
    }	
    
    
    if (!(rcfile = fopen(buf, "rt"))) {
      perror(buf);
      return;
    }
    
	
    while ((ptr = get_next_line(rcfile))) {
      linenumber++;
      if (*ptr == '\0') 
		continue;
      
	  
      if (sscanf(ptr, "%s %s", field, argument) != 2) {
		printf("Error line %d : %s\n", linenumber, ptr);
		continue;
      }
      
      *(field + strlen(field) - 1) = '\0';
      
      for (iline = 0; options[iline].label != NULL; iline ++) {
		if (strcasecmp(field, options[iline].label) )
		  continue;
		
		switch (options[iline].type) {
		case VAL_ONOFF:
		  if (!strcasecmp(argument, "on")) 
			*(bool *) ((options[iline].var)) = TRUE;
		  else
			*((bool*) options[iline].var) = FALSE;
		  continue;
		  break;
		  
		case VAL_INT:
		  *((int *) options[iline].var) = atoi(argument);
		  continue;
		  break;
		  
		case VAL_STRING:
		  strcpy((char *) options[iline].var, argument);
		  continue;
		  break;
		  
		default:
		  break;
		}
		
		if (options[iline].label == NULL) {
		  printf("Unknown option [Line=%d]: %s\n", linenumber, field);
		  continue;
		}
      }
    }     
	
    return;
}




void save_rc() {
  FILE *rcfile;
  char buf[128];
  int iline;

  sprintf(buf, "%s/.cdlabelrc", getenv("HOME"));

  if (!(rcfile = fopen(buf, "wt"))) {
    perror(buf);
    return;
  }
  
  for (iline = 0; options[iline].label != NULL; iline++) {
    switch (options[iline].type) {

    case VAL_ONOFF:
      fprintf(rcfile, "%s: %s\n", options[iline].label,
	      (* (bool*) options[iline].var) == TRUE ? "on" : "off");
      break;
      
    case VAL_INT:
      fprintf(rcfile, "%s: %d\n", options[iline].label,
	      (*(int*) (options[iline].var)));
      break;
      
    case VAL_STRING:
      fprintf(rcfile, "%s: %s\n", options[iline].label,
	      ((char *) (options[iline].var)));
      break;
    }
  }	
  fclose(rcfile); 
}


